' Floating point test program

' This program illustrates the technique of working with numbers in floating point
' format without converting them to integers.  Conversions are performed on the 
' inputs and the final results, but not on the intermediate steps of the calculation.
' This allows you to make calculations where some intermediate steps result in values
' far beyond the 16-bit integer limit. The end result is still subject to this limit.

' Include only one of the following:

	include "fp20.bas"	' Include file for 14-bit core with RAM at $20 (24-bit)
'	include "fp2032.bas"	' Include file for 14-bit core with RAM at $20 (32-bit)
'	include "fp40.bas"	' Include file for 14-bit core with RAM at $40 (24-bit)
'	include "fp4032.bas"	' Include file for 14-bit core with RAM at $40 (32-bit)
'       include "fp17.bas"      ' Include file for 17Cxxx (24-bit)
'       include "fp1732.bas"    ' Include file for 17Cxxx (32-bit)
'	include "fp18.bas"	' Include file for 18Xxxx (24-bit)
'	include "fp1832.bas"	' Include file for 18Xxxx (32-bit)

' Define LCD registers and bits
Define  LCD_DREG        PORTD
Define  LCD_DBIT        4
Define  LCD_RSREG       PORTE
Define  LCD_RSBIT       0
Define  LCD_EREG        PORTE
Define  LCD_EBIT        1

' Define the target device for PM assembler
Define  __16F877        1       ' Only use for PM, MPASM will define this by itself

cdproductexp	var	byte		'4 bytes required for storing numbers
cdproduct0		var	byte		'in floating point format
cdproduct1		var	byte
cdproduct2		var	byte

ainput			con	1005		'4 inputs - A,B,C, and D
binput			con	1000
cinput			con	1000
dinput			con	1000


		ADCON1 = 7      		' Set PORTA and PORTE to digital
		Low PORTE.2     		' LCD R/W line low (W)
		Pause 500       		' Wait for LCD to start


' Demonstrate floating point (A*B - C*D)
' multiply C*D using float (1000*1000 = 1005000)

        aint = cinput
        Gosub itofa     		' Convert int to float

        bint = dinput
        Gosub itofb     		' Convert int to float

        Gosub fpmul     		' FP multiply

		cdproductexp = aexp		' store the product of C*D in 
		cdproduct0 = aargb0		' floating point format
		cdproduct1 = aargb1
		cdproduct2 = aargb2

' multiply A*B using float (1005 * 1000 = 1005000)

		aint = ainput
		Gosub itofa

		bint = binput
		Gosub itofb

		gosub fpmul

		bexp = cdproductexp		' restore the product of C*D in
		bargb0 = cdproduct0		' floating point format (B arg)
		bargb1 = cdproduct1
		bargb2 = cdproduct2

		Gosub fpsub				' perform the subtraction A*B - C*D (1005000 - 1000000 = 5000)

        Gosub ftoia     		' Convert float to int


        Lcdout $fe, 1, "(A*B)-(C*D) = ", dec aint		' display the end result in decimal

		End
